#!/bin/bash

. /etc/default/${EPAGES_PROJECT:-epages6}
PERL_EXE=$EPAGES_PERL/bin/perl

ERROR=$(ls Makefile.PL patch.pl $PERL_EXE 2>&1 >/dev/null)
RESULT=$?
if [[ $RESULT != 0 ]] ; then
  echo "patch.sh: $ERROR"
  exit $RESULT
fi

hostname=$(hostname)
PATCH_TAG=$(awk -F= '/^PATCH_TAG[[:space:]]*=/{gsub(/[[:space:]]+/,""); print $2}' Makefile.PL)
LOG_DIR=${EPAGES_PATCHES:-"$EPAGES_SHARED/Patches"}/var-$PATCH_TAG/logs/$hostname
mkdir -m 755 -p $LOG_DIR || exit 1

#------------------------------------------------------------------------------
# change perl
#------------------------------------------------------------------------------

unset PERL_HAS_CHANGED

grep -q 'EPAGES_CARTRIDGES=$2' $PERL_EXE || [[ ! -f sh/6017005000/EPAGES_PERL/bin/perl ]] || {
  # $EPAGES_PERL/bin/perl is outdated and new is available
  cat sh/6017005000/EPAGES_PERL/bin/perl > $PERL_EXE || exit 1
}

# do not update perl if perl flex server is running
service ePagesConfigServer status 2>/dev/null && SKIP_PERL_UPDATE=1 || SKIP_PERL_UPDATE=

# update perl if libssl.so is outdated and new is available
LIBSSL=lib32/ssl/libssl.so
grep -q '1\.0\.0m' $EPAGES_PERL/$LIBSSL || [[ ! -f sh/6017006000/EPAGES_PERL/$LIBSSL ]] || {
  PERL_HAS_CHANGED=
  for i in $(ls -d sh/6017006000/EPAGES_PERL/lib32/ssl/*.so* 2>/dev/null) ; do
    # update library here
    [[ -z $SKIP_PERL_UPDATE ]] && cat $i > $EPAGES_PERL/${i##*/EPAGES_PERL/}
    PERL_HAS_CHANGED=1
    # therefore delete entry of library in sh/6017006000/patch.conf
    sed -i "s,^${i##*/EPAGES_PERL/}=.*,," sh/6017006000/patch.conf
  done
  # sh/6017006000/patch.conf has changed -> delete entry in filelist.txt
  if [[ -n $PERL_HAS_CHANGED && -f filelist.txt ]] ; then
    cp -p filelist.txt filelist.orig
    grep -v sh/6017006000/patch.conf filelist.orig > filelist.txt
  fi
  # and remember this
  [[ -z $SKIP_PERL_UPDATE && -n $PERL_HAS_CHANGED ]] && touch ${LOG_DIR%/*}/PERL_HAS_CHANGED
}

if [[ -f ${LOG_DIR%/*}/PERL_HAS_CHANGED && ! -f $LOG_DIR/PERL_WAS_RESTARTED ]] ; then
  # due to changed binaries we must restart the application server
  if [[ -x ${EPSERVICE:-"$EPAGES/Service/epagesService"} ]] ; then
    EPAGES_IGNORE=cron /etc/init.d/${EPAGES_PROJECT:-epages6} start_service
  fi
  touch $LOG_DIR/PERL_WAS_RESTARTED
fi

#------------------------------------------------------------------------------
# run perl
#------------------------------------------------------------------------------

date=$(date +'%Y-%h-%d-%H.%M.%S')
$EPAGES_PERL/bin/perl patch.pl "$@" 2>&1 | tee $LOG_DIR/$date-${1////-}.log

# return success or failure
RESULT=${PIPESTATUS[0]}
echo "$0 $@: $RESULT" >> /tmp/epages-$PATCH_TAG-result.log

exit $RESULT
